==> xlr-server-sql-24.1.3-sources.jar: com/xebialabs/xlrelease/repository/sql/persistence/ReleaseCacheService.scala <==
package com.xebialabs.xlrelease.repository.sql.persistence

import com.github.benmanes.caffeine.cache.Cache
import com.xebialabs.xlrelease.config.XlrConfig
import com.xebialabs.xlrelease.support.config.TypesafeConfigExt._
import com.xebialabs.xlrelease.repository.sql.persistence.CiId._
import com.xebialabs.xlrelease.repository.sql.persistence.data.ReleaseRow
import com.xebialabs.xlrelease.service.FeatureService
import com.xebialabs.xlrelease.support.cache.caffeine.CaffeineCacheBuilder
import grizzled.slf4j.Logging
import org.springframework.jmx.`export`.annotation.{ManagedOperation, ManagedResource}

import java.util.concurrent.ConcurrentHashMap

@ManagedResource(objectName = "com.xebialabs.xlrelease.settings:name=ReleaseCacheService", description = "MBean to configure internal release cache")
sealed trait ReleaseCacheService extends FeatureService {

  override def name(): CiId = "ReleaseCacheService"

  @ManagedOperation(description = "Invalidate provided entry with provided Release ID as a key")
  def invalidate(releaseId: CiId): Unit

  @ManagedOperation(description = "Invalidate all entries")
  def invalidateAll(): Unit

  @ManagedOperation(description = "Get cache statistics as a string")
  def getStats(): String

}

object ReleaseCacheService extends ReleaseCacheService with Logging with CaffeineCacheBuilder {

  private val cacheName = "release-rows"
  private val cacheConf = XlrConfig.getInstance.cache.config

  private val allowedReleases = new ConcurrentHashMap[String, String]()

  @volatile
  private var enabled: Boolean = {
    val releaseRowCacheEnabled = cacheConf.getConfig(cacheName).getOptionalBoolean("enabled").getOrElse(false)
    releaseRowCacheEnabled
  }

  private val releaseCache: Cache[CiId, ReleaseRow] = {
    val cacheSettings = XlrConfig.getInstance.cache
    buildCache("releaseCacheService", cacheName, "default-cache-values", cacheSettings.config)
  }


  def get(releaseId: CiId): Option[ReleaseRow] = {
    if (enabled) {
      Option(releaseCache.getIfPresent(releaseId.shortId)).map { r: ReleaseRow =>
        logger.trace(s"Fetching ${releaseId.shortId} from cache")
        r
      }
    } else {
      None
    }
  }

  def put(releaseId: CiId, releaseRow: ReleaseRow): Unit = {
    if (enabled) {
      if (allowedReleases.containsKey(releaseId.shortId)) {
        logger.trace(s"Caching ${releaseId.shortId}")
        releaseCache.put(releaseId.shortId, releaseRow)
      }
    }
  }

  def allow(releaseId: CiId): Unit = {
    allowedReleases.putIfAbsent(releaseId.shortId, releaseId)
  }

  def disallow(releaseId: CiId): Unit = {
    allowedReleases.remove(releaseId.shortId)
    invalidate(releaseId)
  }

  override def invalidate(releaseId: CiId): Unit = {
    releaseCache.invalidate(releaseId.shortId)
  }

  override def invalidateAll(): Unit = {
    releaseCache.invalidateAll()
  }

  override def enable(): Unit = {
    enabled = true
  }

  override def disable(): Unit = {
    enabled = false
    releaseCache.invalidateAll()
  }

  def isEnabled(): Boolean = {
    enabled
  }

  override def getStats(): String = {
    val cacheStats = releaseCache.stats()
    cacheStats.toString
  }
}

==> xlr-server-sql-25.1.0-1107.113-sources.jar: com/xebialabs/xlrelease/repository/sql/persistence/ReleaseCacheService.scala <==
package com.xebialabs.xlrelease.repository.sql.persistence

import com.github.benmanes.caffeine.cache.Cache
import com.xebialabs.xlrelease.repository.sql.persistence.CiId._
import com.xebialabs.xlrelease.repository.sql.persistence.data.ReleaseRow
import com.xebialabs.xlrelease.support.cache.caffeine.spring.XlrCaffeineCacheManager
import grizzled.slf4j.Logging
import org.springframework.jmx.`export`.annotation.{ManagedOperation, ManagedResource}

import java.util.concurrent.ConcurrentHashMap

@ManagedResource(objectName = "com.xebialabs.xlrelease.settings:name=ReleaseCacheService", description = "MBean to configure internal release cache")
sealed trait ReleaseCacheServiceMBean {

  @ManagedOperation(description = "Invalidate provided entry with provided Release ID as a key")
  def invalidate(releaseId: CiId): Unit

  @ManagedOperation(description = "Invalidate all entries")
  def invalidateAll(): Unit

  @ManagedOperation(description = "Get cache statistics as a string")
  def getStats(): String

}

class ReleaseCacheService(cacheManager: XlrCaffeineCacheManager) extends ReleaseCacheServiceMBean with Logging {

  private val allowedReleases = new ConcurrentHashMap[String, String]()
  private val cacheName = "release-rows"

  def getCache: Cache[CiId, ReleaseRow] = {
    cacheManager.getOrCreateCache(cacheName)
  }

  def get(releaseId: CiId): Option[ReleaseRow] = {
    Option(getCache.getIfPresent(releaseId.shortId)).map { r: ReleaseRow =>
      logger.trace(s"Fetching ${releaseId.shortId} from cache")
      r
    }
  }

  def put(releaseId: CiId, releaseRow: ReleaseRow): Unit = {
    if (allowedReleases.containsKey(releaseId.shortId)) {
      logger.trace(s"Caching ${releaseId.shortId}")
      getCache.put(releaseId.shortId, releaseRow)
    }
  }

  def allow(releaseId: CiId): Unit = {
    allowedReleases.putIfAbsent(releaseId.shortId, releaseId)
  }

  def disallow(releaseId: CiId): Unit = {
    allowedReleases.remove(releaseId.shortId)
    invalidate(releaseId)
  }

  override def invalidate(releaseId: CiId): Unit = {
    getCache.invalidate(releaseId.shortId)
  }

  override def invalidateAll(): Unit = {
    getCache.invalidateAll()
  }

  override def getStats(): String = {
    val cacheStats = getCache.stats()
    cacheStats.toString
  }
}
